package com.tzld.piaoquan.ad.engine.commons.score.model;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.TreeMap;
import java.util.stream.Collectors;

public class CvrAdjustingModel extends Model {

    private static final Logger LOGGER = LoggerFactory.getLogger(CvrAdjustingModel.class);

    private Table<Double, Double, Double> table = HashBasedTable.create();

    @Override
    public int getModelSize() {
        return table.size();
    }

    @Override
    public boolean loadFromStream(InputStreamReader in) throws Exception {
        Table<Double, Double, Double> initTable = HashBasedTable.create();
        try (BufferedReader input = new BufferedReader(in)) {
            String line;
            int cnt = 0;
            Map<Double, Double> initModel = new TreeMap<>();
            while ((line = input.readLine()) != null) {
                String[] items = line.split("\t");
                if (items.length < 4) {
                    continue;
                }

                double key = new BigDecimal(items[2]).doubleValue();
                double value = new BigDecimal(items[3]).doubleValue();
                initModel.put(key, value);
            }

            // 最终生成的格式为  区间最小值，区间最大值，系数
            List<Double> keySet = initModel.keySet().stream().sorted().collect(Collectors.toList());
            double preKey = 0.0;
            for (Double key : keySet) {
                initTable.put(preKey, key, initModel.get(key));
                preKey = key;
            }
            initTable.put(preKey, Double.MAX_VALUE, initModel.get(preKey));

            this.table = initTable;

            for (Table.Cell<Double, Double, Double> cell : this.table.cellSet()) {
                LOGGER.info("cell.row: {}, cell.column: {}, cell.value: {}", cell.getRowKey(), cell.getColumnKey(), cell.getValue());
            }
            
            LOGGER.info("[CvrAdjustingModel] model load over and size {}", cnt);
        } catch (
                Exception e) {
            LOGGER.info("[CvrAdjustingModel] model load error ", e);
        } finally {
            in.close();

        }
        return true;
    }

    public Double getAdjustingCoefficien(double score) {
        if (Objects.isNull(table)) {
            return 1.0;
        }

        for (Table.Cell<Double, Double, Double> cell : table.cellSet()) {
            double rowKey = cell.getRowKey();
            double columnKey = cell.getColumnKey();
            if (rowKey <= score & score < columnKey) {
                LOGGER.info("score {} in {} - {} , value is {}", score, rowKey, columnKey, cell.getValue());
                return cell.getValue();
            }
        }

        return 1.0;
    }
}
