package com.tzld.piaoquan.ad.engine.commons.score.model;


import com.tzld.piaoquan.ad.engine.commons.util.CompressUtil;
import com.tzld.piaoquan.ad.engine.commons.util.PropertiesUtil;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassificationModel;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.spark.ml.linalg.Vector;
import org.apache.spark.ml.linalg.Vectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Map;


public class XGBoostModel extends Model {
    private static final Logger LOGGER = LoggerFactory.getLogger(XGBoostModel.class);
    private XGBoostClassificationModel model;

    private String[] features = {
            "cpa",
            "b2_1h_ctr",
            "b2_1h_ctcvr",
            "b2_1h_cvr",
            "b2_1h_conver",
            "b2_1h_click",
            "b2_1h_conver*log(view)",
            "b2_1h_conver*ctcvr",
            "b2_2h_ctr",
            "b2_2h_ctcvr",
            "b2_2h_cvr",
            "b2_2h_conver",
            "b2_2h_click",
            "b2_2h_conver*log(view)",
            "b2_2h_conver*ctcvr",
            "b2_3h_ctr",
            "b2_3h_ctcvr",
            "b2_3h_cvr",
            "b2_3h_conver",
            "b2_3h_click",
            "b2_3h_conver*log(view)",
            "b2_3h_conver*ctcvr",
            "b2_6h_ctr",
            "b2_6h_ctcvr"
    };

    @Override
    public int getModelSize() {
        if (this.model == null)
            return 0;
        return 1;
    }

    @Override
    public boolean loadFromStream(InputStreamReader in) throws Exception {
        return false;
    }

    public void cleanModel() {
        this.model = null;
    }

    public Float score(Map<String, String> featureMap) {

        double[] values = new double[features.length];
        for (int i = 0; i < features.length; i++) {
            double v = NumberUtils.toDouble(featureMap.getOrDefault(features[i], "0.0"), 0.0);
            values[i] = v;
        }

        Vector v = Vectors.dense(values);
        double score = model.predict(v);
        return (float) score;
    }

    @Override
    public boolean loadFromStream(InputStream in) throws Exception {
        String modelDir = PropertiesUtil.getString("model.xgboost.path");
        CompressUtil.decompressGzFile(in, modelDir);
        XGBoostClassificationModel model2 = XGBoostClassificationModel.load("file://" + modelDir);
        model2.setMissing(0.0f);
        this.model = model2;
        return true;
    }

}
